package controllers

import (
	"encoding/json"
	"eta/eta_chart_lib/utils"
	"fmt"
	"github.com/beego/beego/v2/server/web"
	"net/http"
	"net/url"
	"strings"
)

type BaseCommonController struct {
	web.Controller
	Lang string `description:"当前语言类型，中文：zh；英文：en；默认：zh"`
}

func (c *BaseCommonController) Prepare() {
	var requestBody string
	method := c.Ctx.Input.Method()
	if method == "GET" {
		requestBody = c.Ctx.Request.RequestURI
	} else {
		requestBody, _ = url.QueryUnescape(string(c.Ctx.Input.RequestBody))
	}
	// 当前语言
	{
		lang := c.Ctx.Input.Header("Lang")
		if lang == "" {
			lang = utils.ZhLangVersion
		}
		c.Lang = lang
	}

	ip := c.Ctx.Input.IP()
	utils.ApiLog.Info("uri:%s, requestBody:%s, responseBody:%s, ip:%s", c.Ctx.Input.URI(), requestBody, ip)
}

func (c *BaseCommonController) ServeJSON(encoding ...bool) {
	var (
		hasIndent   = false
		hasEncoding = false
	)
	if web.BConfig.RunMode == web.PROD {
		hasIndent = false
	}
	if len(encoding) > 0 && encoding[0] == true {
		hasEncoding = true
	}
	if c.Data["json"] == nil {
		go utils.SendEmail("异常提醒:"+utils.RunMode, "接口:"+"URI:"+c.Ctx.Input.URI()+";无返回值", utils.EmailSendToUsers)
		return
	}
	c.JSON(c.Data["json"], hasIndent, hasEncoding)
}

func (c *BaseCommonController) JSON(data interface{}, hasIndent bool, coding bool) error {
	c.Ctx.Output.Header("Content-Type", "application/json; charset=utf-8")
	var content []byte
	var err error
	if hasIndent {
		content, err = json.MarshalIndent(data, "", "  ")
	} else {
		content, err = json.Marshal(data)
	}
	if err != nil {
		http.Error(c.Ctx.Output.Context.ResponseWriter, err.Error(), http.StatusInternalServerError)
		return err
	}
	ip := c.Ctx.Input.IP()
	params := c.Ctx.Input.Params()
	fmt.Println("params")
	fmt.Println(params)
	requestBody, _ := url.QueryUnescape(string(c.Ctx.Input.RequestBody))
	c.logUri(content, requestBody, ip)
	if coding {
		content = []byte(utils.StringsToJSON(string(content)))
	}
	return c.Ctx.Output.Body(content)
}

func (c *BaseCommonController) logUri(content []byte, requestBody, ip string) {
	authorization := ""
	method := c.Ctx.Input.Method()
	uri := c.Ctx.Input.URI()
	fmt.Println("Url:", uri)
	if method != "HEAD" {
		if method == "POST" || method == "GET" {
			authorization = c.Ctx.Input.Header("authorization")
			if authorization == "" {
				authorization = c.Ctx.Input.Header("Authorization")
			}
			if authorization == "" {
				newAuthorization := c.GetString("authorization")
				if newAuthorization != `` {
					authorization = "authorization=" + newAuthorization
				} else {
					newAuthorization = c.GetString("Authorization")
					authorization = "authorization=" + newAuthorization
				}
			} else {
				if strings.Contains(authorization, ";") {
					authorization = strings.Replace(authorization, ";", "$", 1)
				}
			}
			if authorization == "" {
				strArr := strings.Split(uri, "?")
				for k, v := range strArr {
					fmt.Println(k, v)
				}
				if len(strArr) > 1 {
					authorization = strArr[1]
					authorization = strings.Replace(authorization, "Authorization", "authorization", -1)
					fmt.Println(authorization)
				}
			}
		}
	}

	utils.ApiLog.Info("uri:%s, authorization:%s, requestBody:%s, responseBody:%s, ip:%s", c.Ctx.Input.URI(), authorization, requestBody, content, ip)
	return
}
