package cross_variety

import (
	"eta/eta_chart_lib/utils"
	"github.com/beego/beego/v2/client/orm"
	"time"
)

// ChartTagVariety
// @Description: chart_tag_variety 图表标签品种关系表
type ChartTagVariety struct {
	Id                        int       `orm:"column(id);pk"`
	ChartTagId                int       `description:"标签id"`
	ChartVarietyId            int       `description:"品种id"`
	EdbInfoId                 int       `description:"指标id"`
	LastUpdateSysUserId       int       `description:"最后一次操作人"`
	LastUpdateSysUserRealName string    `description:"最后一次操作人真实姓名"`
	ModifyTime                time.Time `description:"修改时间"`
	CreateTime                time.Time `description:"创建时间"`
}

// GetChartTagVarietyListByTagAndVariety
// @Description: 根据标签id和品种id列表获取所绑定的品种列表
// @author: Roc
// @datetime 2023-11-23 15:04:20
// @param chartTagId int
// @param varietyIdList []int
// @return items []*ChartTagVariety
// @return err error
func GetChartTagVarietyListByTagAndVariety(chartTagId int, varietyIdList []int) (items []*ChartTagVariety, err error) {
	num := len(varietyIdList)
	if num <= 0 {
		return
	}
	o := orm.NewOrmUsingDB("data")
	sql := `SELECT * FROM chart_tag_variety WHERE chart_tag_id = ? AND chart_variety_id in (` + utils.GetOrmInReplace(num) + `) `
	_, err = o.Raw(sql, chartTagId, varietyIdList).QueryRows(&items)

	return
}
