package models

import (
	"time"

	"github.com/beego/beego/v2/client/orm"
)

type ExcelInfoRuleMapping struct {
	ExcelInfoRuleMappingId int       `orm:"pk" description:"主键"`
	ExcelInfoId            int       `description:"Excel信息ID"`
	RuleType               int       `description:"规则类型"`
	LeftValue              string    `description:"左值"`
	LeftValueShow          string    `description:"左值前端显示"`
	LeftValueType          int       `description:"左值类型"`
	RightValue             string    `description:"右值"`
	RightValueShow         string    `description:"右值前端显示"`
	RightValueType         int       `description:"右值类型"`
	FontColor              string    `description:"字体颜色"`
	BackgroundColor        string    `description:"背景颜色"`
	Remark                 string    `description:"预设颜色说明"`
	RemarkEn               string    `description:"预设颜色英文说明"`
	Scope                  string    `description:"作用范围"`
	ScopeCoord             string    `description:"作用范围坐标"`
	ScopeShow              string    `description:"作用范围坐标前端显示"`
	CreateTime             time.Time `description:"创建时间"`
}

type ExcelInfoRuleMappingView struct {
	ExcelInfoRuleMappingId int    `orm:"pk" description:"主键"`
	ExcelInfoId            int    `description:"Excel信息ID"`
	RuleType               int    `description:"规则类型：1-大于，2-小于，3-介于，4-等于，5-发生日期"`
	LeftValue              string `description:"左值"`
	LeftValueBack          string `description:"左值前端显示"`
	LeftValueType          int    `description:"左值类型"`
	RightValue             string `description:"右值"`
	RightValueBack         string `description:"右值前端显示"`
	RightValueType         int    `description:"右值类型"`
	FontColor              string `description:"字体颜色"`
	BackgroundColor        string `description:"背景颜色"`
	Remark                 string `description:"预设颜色说明"`
	RemarkEn               string `description:"预设颜色英文说明"`
	Scope                  string `description:"作用范围"`
	ScopeCoord             string `description:"作用范围坐标"`
	ScopeShow              string `description:"作用范围坐标前端显示"`
	CreateTime             string `description:"创建时间"`
}

func (e *ExcelInfoRuleMapping) Insert() (insertId int64, err error) {
	o := orm.NewOrmUsingDB("data")
	insertId, err = o.Insert(e)
	return
}

func (e *ExcelInfoRuleMapping) Update(cols []string) (err error) {
	o := orm.NewOrmUsingDB("data")
	_, err = o.Update(e, cols...)
	return
}

// GetExcelRuleMappingByExcelInfoId 根据excelInfoId获取规则映射信息
func GetExcelRuleMappingByExcelInfoId(id int) (items []*ExcelInfoRuleMappingView, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `SELECT * FROM excel_info_rule_mapping WHERE excel_info_id = ? ORDER BY create_time ASC`
	_, err = o.Raw(sql, id).QueryRows(&items)
	return
}

// GetExcelRuleMappingById 根据主键Id获取规则映射信息
func GetExcelRuleMappingById(id int) (item *ExcelInfoRuleMappingView, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `SELECT * FROM excel_info_rule_mapping WHERE excel_info_rule_mapping_id = ?`
	err = o.Raw(sql, id).QueryRow(&item)
	return
}

func DeleteExcelRuleMappingById(id int) (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `DELETE FROM excel_info_rule_mapping WHERE excel_info_rule_mapping_id = ?`
	_, err = o.Raw(sql, id).Exec()
	return
}
