package utils

import (
	"encoding/json"
	"github.com/beego/beego/v2/core/logs"
	"os"
	"path"
)

const (
	DefaultLogPath    = "./etalogs/filelog"
	DefaultBinlogPath = "./etalogs/binlog"
	DefaultApiLogPath = "./etalogs/apilog"
)

var FileLog *logs.BeeLogger
var ApiLog *logs.BeeLogger
var Binlog *logs.BeeLogger
var MongoLog *logs.BeeLogger

func init() {
	if LogMaxDays == 0 {
		LogMaxDays = 30
	}
	logPath := LogPath
	if logPath == "" {
		logPath = DefaultLogPath
	}
	logFile := LogFile
	if logFile == "" {
		logFile = "filelog.log"
	}
	os.MkdirAll(logPath, os.ModePerm)

	// 打开文件
	logFileName := path.Join(logPath, logFile)
	FileLog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()

	logConf.FileName = logFileName
	b, _ := json.Marshal(logConf)
	FileLog.SetLogger(logs.AdapterFile, string(b))
	FileLog.EnableFuncCallDepth(true)

	initBinlog()
	initApiLog()
	initMgoLog()
}

type logConfig struct {
	FileName string `json:"filename" description:"保存的文件名"`
	MaxLines int    `json:"maxlines"  description:"每个文件保存的最大行数，默认值 1000000"`
	MaxSize  int    `json:"maxsize" description:"每个文件保存的最大尺寸，默认值是 1 << 28, //256 MB"`
	Daily    bool   `json:"daily" description:"是否按照每天 logrotate，默认是 true"`
	MaxDays  int    `json:"maxdays" description:"文件最多保存多少天，默认保存 7 天"`
	Rotate   bool   `json:"rotate" description:"是否开启 logrotate，默认是 true"`
	Level    int    `json:"level" description:"日志保存的时候的级别，默认是 Trace 级别"`
	Color    bool   `json:"color" description:"日志是否输出颜色"`
	//Perm     string `json:"perm" description:"日志文件权限"`
}

func initBinlog() {
	//binlog日志
	//binlog日志
	binlogPath := BinLogPath
	if binlogPath == "" {
		binlogPath = DefaultBinlogPath
	}
	binlogFile := BinLogFile
	if binlogFile == "" {
		binlogFile = "binlog.log"
	}
	os.MkdirAll(binlogPath, os.ModePerm)
	logFileName := path.Join(binlogPath, binlogFile)
	Binlog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()

	logConf.FileName = logFileName
	//logConf.MaxLines = 10000000
	//logConf.Rotate = true
	b, _ := json.Marshal(logConf)
	Binlog.SetLogger(logs.AdapterFile, string(b))
	Binlog.EnableFuncCallDepth(true)
}

func initApiLog() {
	logPath := ApiLogPath
	if logPath == "" {
		logPath = DefaultApiLogPath
	}
	logFile := ApiLogFile
	if logFile == "" {
		logFile = "apilog.log"
	}
	os.MkdirAll(logPath, os.ModePerm)

	// 打开文件
	logFileName := path.Join(logPath, logFile)
	ApiLog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()

	logConf.FileName = logFileName
	b, _ := json.Marshal(logConf)
	ApiLog.SetLogger(logs.AdapterFile, string(b))
	ApiLog.EnableFuncCallDepth(true)
}

// initMgoLog
// @Description: mongo日志
// @author: Roc
// @datetime 2024-05-06 16:42:47
func initMgoLog() {
	//mgo日志

	var logPath string
	{
		binlogPath := BinLogPath
		if binlogPath == "" {
			binlogPath = DefaultBinlogPath
		}
		logPath = path.Dir(binlogPath)
	}
	if logPath == `` {
		logPath = "./etalogs"
	}

	mongoLogPath := logPath + "/mongolog"
	mongoLogFile := "mongolog.log"

	os.MkdirAll(mongoLogPath, os.ModePerm)
	logFileName := path.Join(mongoLogPath, mongoLogFile)
	MongoLog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()

	logConf.FileName = logFileName
	//logConf.MaxLines = 10000000
	//logConf.Rotate = true
	b, _ := json.Marshal(logConf)
	MongoLog.SetLogger(logs.AdapterFile, string(b))
	MongoLog.EnableFuncCallDepth(true)
}

func getDefaultLogConfig() logConfig {
	return logConfig{
		FileName: "",
		MaxLines: 10000000,
		MaxSize:  1 << 28,
		Daily:    true,
		MaxDays:  LogMaxDays, //我就是喜欢31天，咋滴，不喜欢你就自己改-_-!
		Rotate:   true,
		Level:    logs.LevelTrace,
		//Perm:     "",
	}
}
